#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.program = void 0;
const path = require("path");
const fs = require("fs");
const commander = require("commander");
const write_1 = require("../util/write");
const Session_js_1 = require("../runner/Session.js");
const loadParameters_1 = require("./loadParameters");
const packageData = JSON.parse(fs.readFileSync(path.join(__dirname, '../../package.json'), 'utf8'));
exports.program = new commander.Command()
    .version(packageData.version)
    .usage('[options] <include ...>')
    .option('--helper <path>', 'A path where the helper script will be output. You can\'t use --helper with --css.')
    .option('--css <path>', 'A path where the css will be output. You can\'t use --css with --helper.')
    .option('--ext <ext>', 'An extension of scripts generated from css.')
    .option('--config <path>', 'A path to configuration files.')
    .option('--exclude <path ...>', 'Paths or patterns to be excluded.')
    .option('--noMangle', 'Keep the original name for debugging.')
    .option('--watch', 'Watch files and update the modules automatically.');
if (!module.parent) {
    exports.program.parse(process.argv);
    loadParameters_1.loadParameters(exports.program)
        .then(async (parameters) => {
        await new Session_js_1.Session(parameters).start();
    })
        .catch((error) => {
        write_1.write(process.stderr, [error]);
        process.exit(1);
    });
}