"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findAddStyleImport = exports.normalizeHelperId = void 0;
const path = require("path");
const ast_1 = require("./ast");
exports.normalizeHelperId = (id) => path.normalize(id).replace(/\.ts$/, '');
exports.findAddStyleImport = (ast, helperId, localName) => {
    if (!ast_1.isProgramNode(ast)) {
        throw new Error('InvalidNode');
    }
    for (const node of ast.body) {
        if (node.type === 'ImportDeclaration') {
            if (exports.normalizeHelperId(node.source.value) === exports.normalizeHelperId(helperId)) {
                const { specifiers = [] } = node;
                if (specifiers.length === 1) {
                    return { name: specifiers[0].local.name, node };
                }
            }
        }
    }
    if (localName) {
        for (const node of ast.body) {
            switch (node.type) {
                case 'VariableDeclaration': {
                    const { declarations } = node;
                    if (declarations.length === 1 && declarations[0].id.name === localName) {
                        return { name: localName, node };
                    }
                    break;
                }
                case 'FunctionDeclaration':
                    if (node.id.name === localName) {
                        return { name: localName, node };
                    }
                    break;
                default:
            }
        }
    }
    throw new Error('NoAddStyle');
};