"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTransformer = void 0;
const transformDeclarations_1 = require("./transformDeclarations");
const getImports_1 = require("./getImports");
const minify_1 = require("./minify");
const mangleIdentifiers_1 = require("./mangleIdentifiers");
const mangleKeyFrames_1 = require("./mangleKeyFrames");
const removeImportsAndRaws_1 = require("./removeImportsAndRaws");
const normalizePath_1 = require("../util/normalizePath");
exports.createTransformer = ({ mangler, rawPrefix }) => async (root, result) => {
    const id = normalizePath_1.normalizePath((result.opts && result.opts.from) || Date.now().toString(36));
    const imports = getImports_1.getImports(root, id);
    const transformResult = {
        ...(await mangleIdentifiers_1.mangleIdentifiers({ id, root, mangler, imports, rawPrefix })),
        keyframes: mangleKeyFrames_1.mangleKeyFrames({ id, root, mangler, imports, rawPrefix }),
    };
    transformDeclarations_1.transformDeclarations({ root, mangler, imports, transformResult, rawPrefix });
    minify_1.minify(root);
    return removeImportsAndRaws_1.removeImportsAndRaws({
        result: transformResult,
        imports,
        rawPrefix,
    });
};