"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mangleIdentifiers = void 0;
const selectorParser = require("postcss-selector-parser");
const getMatchedImport_1 = require("./getMatchedImport");
exports.mangleIdentifiers = async ({ id, root, mangler, imports, rawPrefix }) => {
    const result = {
        id: {},
        className: {},
    };
    const parser = selectorParser((selectors) => {
        selectors.walk((selector) => {
            if (selectorParser.isClassName(selector) || selectorParser.isIdentifier(selector)) {
                const { value: before } = selector;
                let after = before;
                if (before.startsWith(rawPrefix)) {
                    after = before.slice(rawPrefix.length);
                }
                else {
                    const imported = getMatchedImport_1.getMatchedImport(before, imports);
                    if (imported) {
                        after = mangler(imported.from, selector.type, imported.key);
                    }
                    else {
                        after = mangler(id, selector.type, before);
                    }
                }
                const type = selector.type === 'id' ? 'id' : 'className';
                selector.value = result[type][before] = after;
            }
        });
    });
    const processes = [];
    root.walkRules((rule) => {
        processes.push(parser.process(rule.selector).then((newSelector) => {
            rule.selector = newSelector;
        }));
    });
    await Promise.all(processes);
    return result;
};