"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformDeclarations = void 0;
const parser = require("@hookun/parse-animation-shorthand");
const getMatchedImport_1 = require("./getMatchedImport");
exports.transformDeclarations = ({ root, transformResult, mangler, imports, rawPrefix }) => {
    const getRenamed = (name) => {
        if (name.startsWith(rawPrefix)) {
            return name.slice(rawPrefix.length);
        }
        const imported = getMatchedImport_1.getMatchedImport(name, imports);
        if (imported) {
            return mangler(imported.from, 'keyframes', imported.key);
        }
        return transformResult.keyframes[name];
    };
    root.walkDecls((declaration) => {
        const { prop } = declaration;
        if (prop === 'animation-name') {
            const renamed = getRenamed(declaration.value);
            if (renamed) {
                declaration.value = renamed;
            }
        }
        else if (prop === 'animation') {
            const animations = [];
            for (const animation of parser.parse(declaration.value)) {
                const renamed = getRenamed(animation.name);
                if (renamed) {
                    animation.name = renamed;
                }
                animations.push(parser.serialize(animation));
            }
            declaration.value = animations.join(',');
        }
    });
};