/// <reference types="node" />
import * as fs from 'fs';
import * as chokidar from 'chokidar';
import { ISessionOptions, ISessionConfiguration } from './types';
import { write } from '../util/write';
import { IExposedPromise } from '../util/createExposedPromise';
export declare class Session {
    readonly configuration: Readonly<ISessionConfiguration>;
    protected watcher?: chokidar.FSWatcher;
    protected processedFiles: Set<string>;
    protected initialTask: Array<Promise<void>> | null;
    protected tasks: Set<Promise<void>>;
    constructor(parameters?: ISessionOptions);
    get helperPath(): string;
    start(): Promise<void>;
    stop(): Promise<void>;
    outputHelperScript(): Promise<void>;
    processCSS(filePath: string): Promise<{
        dest: string;
        code: string;
    }>;
    minifyScripts(): Promise<void>;
    protected waitCurrentTasks(): Promise<void>;
    protected createExposedPromise(): IExposedPromise;
    protected startWatcher(): Promise<void>;
    protected log(...messages: Parameters<typeof write>[1]): void;
    protected logError(...messages: Parameters<typeof write>[1]): void;
    protected stopWatcher(): Promise<void>;
    protected onError(error: Error): void;
    protected onFileEvent(eventName: 'add' | 'addDir' | 'change' | 'unlink' | 'unlinkDir', file: string, stats?: fs.Stats): Promise<void>;
    protected onAdd(file: string, stats?: fs.Stats): Promise<void>;
    protected onChange(file: string, stats?: fs.Stats): Promise<void>;
    protected onUnlink(file: string): Promise<void>;
}
