/// <reference types="node" />
import * as anymatch from 'anymatch';
import * as chokidar from 'chokidar';
import * as postcss from 'postcss';
import * as stream from 'stream';
import { IPluginOptions } from '../postcssPlugin/types';
export interface ISessionOptions {
    include?: string | Array<string>;
    exclude?: anymatch.Matcher;
    extensions?: Array<string>;
    helper?: string;
    ext?: string;
    css?: string;
    watch?: boolean;
    chokidar?: chokidar.WatchOptions;
    postcssPlugins?: Array<postcss.AcceptedPlugin>;
    postcssOptions?: postcss.ProcessOptions;
    esifycssPluginParameter?: IPluginOptions;
    stdout?: stream.Writable;
    stderr?: stream.Writable;
}
export interface IReadonlyWatchOptions extends Readonly<chokidar.WatchOptions> {
    awaitWriteFinish?: Readonly<chokidar.AwaitWriteFinishOptions> | boolean;
}
export interface ISessionOutput {
    type: 'script' | 'css';
    path: string;
}
export interface ISessionConfiguration {
    readonly watch: boolean;
    readonly output: ISessionOutput;
    readonly ext: string;
    readonly path: ReadonlyArray<string>;
    readonly chokidar: IReadonlyWatchOptions;
    readonly stdout: stream.Writable;
    readonly stderr: stream.Writable;
    readonly postcssPlugins: Array<postcss.AcceptedPlugin>;
    readonly postcssOptions: postcss.ProcessOptions;
    readonly cssKey: string;
}
export interface ICSSParserParameters {
    file: string;
    css?: string | Buffer;
    options?: postcss.ProcessOptions;
    plugins: Array<postcss.AcceptedPlugin>;
    map?: postcss.SourceMapOptions;
}
export interface ICSSParserConfigurations {
    readonly css: string;
    readonly plugins: Array<postcss.AcceptedPlugin>;
    readonly options: {
        readonly from: string;
        readonly map: postcss.SourceMapOptions;
    };
}
