"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteFile = exports.writeFile = exports.copyFile = exports.rmdir = exports.unlink = exports.stat = exports.readFile = exports.mkdir = exports.mkdtemp = exports.readdir = void 0;
const path = require("path");
const fs = require("fs");
const util = require("util");
const createDirectoryFor_1 = require("./createDirectoryFor");
const write_1 = require("./write");
exports.readdir = util.promisify(fs.readdir);
exports.mkdtemp = util.promisify(fs.mkdtemp);
exports.mkdir = util.promisify(fs.mkdir);
exports.readFile = util.promisify(fs.readFile);
exports.stat = util.promisify(fs.stat);
exports.unlink = util.promisify(fs.unlink);
exports.rmdir = util.promisify(fs.rmdir);
exports.copyFile = util.promisify(fs.copyFile);
exports.writeFile = async (dest, data, stdout = process.stdout) => {
    await new Promise((resolve, reject) => {
        createDirectoryFor_1.createDirectoryFor(dest);
        const buffer = Buffer.isBuffer(data) ? data : Buffer.from(data);
        fs.writeFile(dest, buffer, (error) => {
            if (error) {
                reject(error);
            }
            else {
                const isBig = 100000 < buffer.length;
                write_1.write(stdout, [`written: ${dest}${isBig ? ` ${buffer.length}bytes` : ''}`]);
                resolve();
            }
        });
    });
};
exports.deleteFile = async (filePath, stdout = process.stdout) => {
    try {
        await exports.unlink(filePath);
    }
    catch (error) {
        switch (error.code) {
            case 'ENOENT':
                return;
            case 'EISDIR':
            case 'EPERM': {
                const files = (await exports.readdir(filePath)).map((name) => path.join(filePath, name));
                await Promise.all(files.map(async (file) => {
                    await exports.deleteFile(file, stdout);
                }));
                await exports.rmdir(filePath);
                break;
            }
            default:
                throw error;
        }
    }
    write_1.write(stdout, [`deleted: ${filePath}`]);
};